// Simple implementation of a dynamically growing array list; rough
// approximation to ArrayList

public class MiniAL<T> {
  // Default capacity on creation
  private static int defaultSize = 10;

  // How to grow when the internal data store must be expanded
  private static int grow(int size){
    return (size * 3)/2 + 1;
  }
  private T [] data;            // Internal store of objects
  private int size;             // Virtual size

  // Make a default capacity 
  public MiniAL(){
    this(defaultSize);
  }

  // Create a list with indicated initial capacity
  @SuppressWarnings("unchecked") 
  public MiniAL(int n){
    this.data = (T []) new Object[n];
    this.size = 0;
  }
  // Virtual size of the AL
  public int size(){
    return this.size;
  }
    

  // Ensure that the given index is in bounds
  public void assertInBounds(int i){
    if(i < 0 || i>=this.size()){
      String msg = String.format("%d is out of bounds in size %d AL",
                                 i,this.size());
      throw new RuntimeException(msg);
    }
  }

  // Retrieve an element, Normally unsafe to typecaste a generic; use
  // annotation @SuppressWarnings to suppress irritating warnings
  public T get(int i){
    assertInBounds(i);
    return data[i];
  }
  // Set an element at a given position; can result in out of bounds
  // access
  public void set(int i, T x){
    assertInBounds(i);
    data[i] = x;
  }
  // Add a new element, expand if needed
  @SuppressWarnings("unchecked") 
  public void add(T x){
    if(this.size >= this.data.length){
      // Must increase capacity
      T b[] =  (T []) new Object[grow(this.data.length)];
      for(int i=0; i<this.data.length; i++){
  	b[i] = data[i];
      }
      this.data = b;
    }
    data[this.size] = x;
    this.size++;
  }
  // Pretty print
  public String toString(){
    if(this.size()==0){
      return "[]";
    }
    StringBuffer sb = new StringBuffer();
    sb.append("[");
    for(int i=0; i<this.size-1; i++){
      sb.append(this.data[i].toString());
      sb.append(", ");
    }
    sb.append(this.data[this.size-1]);
    sb.append("]");
    return sb.toString();
  }
}


// Alternatives for add()
// 
// // Alternative 1
// Object b[] = new Object[grow(this.a.length)];
// System.arraycopy(a, 0, b, 0, a.length);
// 
// // Alternative 2
// this.a = java.util.Arrays.copyOf(this.a, grow(this.a.length));
